/*
 *  ----------------------------------------------------------------------------
 *  Filename      : Application.java
 *  ----------------------------------------------------------------------------
 *  Application   : Soekia
 *  Author        : Matthias Dreier
 *  File created  : 08.09.2003
 *  ----------------------------------------------------------------------------
 *  This file is part of Soekia.
 *
 *  Soekia is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Soekia is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Soekia; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  ----------------------------------------------------------------------------
 */
package ch.educeth.soekia;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileWriter;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.JWindow;
import javax.swing.ScrollPaneConstants;
import javax.swing.Timer;
import javax.swing.UIManager;
import javax.swing.WindowConstants;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;

import ch.educeth.soekia.index.IndexTask;
import ch.educeth.soekia.lucene.AnalyzerFactory;
import ch.educeth.soekia.util.QueryTerm;
import ch.educeth.soekia.util.RankedDocumentSet;
import ch.educeth.soekia.util.StringConstants;
import ch.educeth.soekia.xml.Setting;
import ch.educeth.soekia.xml.Settings;
import ch.educeth.soekia.xml.XmlExporter;
import ch.educeth.soekia.xml.XmlImporter;

import java.awt.Desktop;
/**
 * Application.
 *
 * @author	Matthias Dreier
 */
public class Application extends JPanel {

	/**
	 * The flag whether the index creation is canceled.
	 */
	public boolean indexCanceled;

	/**
	 * The vector of files.
	 */
	public Vector fileVector;

	/**
	 * The index writer.
	 */
	public IndexWriter indexWriter;

	/**
	 * The flag whether to do stemming.
	 */
	protected boolean doStemming;

	/**
	 * The flag whether to remove the stop words.
	 */
	protected boolean removeStopWords;

	/**
	 * The weight for ranking principle 1.
	 */
	protected int ranking1;

	/**
	 * The weight for ranking principle 2.
	 */
	protected int ranking2;

	/**
	 * The weight for ranking principle 3.
	 */
	protected int ranking3;

	/**
	 * The control for this application.
	 */
	protected Control control;

	/**
	 * The document directory.
	 */
	protected File documentDirectory;

	/**
	 * The index directory.
	 */
	protected File indexDirectory;

	/**
	 * The check box for subdirectories.
	 */
	protected JCheckBox subdirsCheckBox;

	/**
	 * The combo box for the language.
	 */
	protected JComboBox languageCombo;

	/**
	 * The combo box for stemming.
	 */
	protected JComboBox stemmingCombo;

	/**
	 * The combo box for stop words.
	 */
	protected JComboBox stopWordCombo;

	/**
	 * The dialog for stop words.
	 */
	protected JDialog stopWordDialog;

	/**
	 * The text pane for stop words.
	 */
	protected JEditorPane stopWordTextPane;

	/**
	 * The frame for the help window.
	 */
	protected JFrame helpFrame;

	/**
	 * The text field for the document directory.
	 */
	protected JTextField documentField;

	/**
	 * The text field for the index directory.
	 */
	protected JTextField indexField;

	/**
	 * The text field for the query.
	 */
	protected JTextField queryField;

	/**
	 * The string for the language.
	 */
	protected String language;

	/**
	 * The string containing the stop words.
	 */
	protected String stopWords;

	/**
	 * The timer for the progress bar.
	 */
	protected Timer timer;

	/**
	 * The GUI factory.
	 */
	protected GuiFactory guiFactory;

	/**
	 * Creates a new Application instance and adds the GUI components.
	 */
	public Application() {
		control = new Control(this);
		guiFactory = new GuiFactory();

		indexCanceled = false;
		language = StringConstants.getString(StringConstants.LANGUAGE);
		ranking1 = 2;
		ranking2 = 2;
		ranking3 = 2;

		stopWords = "";

		setBackground(guiFactory.backgroundColor);
		createGui();
	}

	/**
	 * Creates the GUI components.
	 */
	protected void createGui() {
		/*
		 * Panels
		 */
		JPanel documentPanel = guiFactory.createJPanel(new BorderLayout());
		documentPanel.setAlignmentX(LEFT_ALIGNMENT);
		documentPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		JPanel indexPanel = guiFactory.createJPanel(new BorderLayout());
		indexPanel.setAlignmentX(LEFT_ALIGNMENT);
		indexPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		JPanel parameterPanel = guiFactory.createJPanel();
		parameterPanel.setAlignmentX(LEFT_ALIGNMENT);
		parameterPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		parameterPanel.setLayout(new BoxLayout(parameterPanel, BoxLayout.Y_AXIS));
		JPanel creationPanel = guiFactory.createJPanel();
		creationPanel.setAlignmentX(LEFT_ALIGNMENT);
		creationPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		creationPanel.setLayout(new BoxLayout(creationPanel, BoxLayout.Y_AXIS));
		JPanel queryPanel = guiFactory.createJPanel();
		queryPanel.setAlignmentX(LEFT_ALIGNMENT);
		queryPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		queryPanel.setLayout(new BoxLayout(queryPanel, BoxLayout.Y_AXIS));

		/*
		 * Document collection panel
		 */
		JButton helpButton = new JButton(Application.getIcon("help.gif"));
		helpButton.setActionCommand("help");
		helpButton.addActionListener(control);
		helpButton.addMouseListener(new MouseAdapter() {
			public void mouseEntered(MouseEvent e) {
				((JButton) e.getSource()).setBorderPainted(true);
			}
			public void mouseExited(MouseEvent e) {
				((JButton) e.getSource()).setBorderPainted(false);
			}
		});
		helpButton.setBorderPainted(false);
		helpButton.setFocusable(false);
		helpButton.setOpaque(false);
		helpButton.setPreferredSize(new Dimension(32, 32));
		JButton aboutButton = new JButton(Application.getIcon("about.gif"));
		aboutButton.setActionCommand("about");
		aboutButton.addActionListener(control);
		aboutButton.addMouseListener(new MouseAdapter() {
			public void mouseEntered(MouseEvent e) {
				((JButton) e.getSource()).setBorderPainted(true);
			}
			public void mouseExited(MouseEvent e) {
				((JButton) e.getSource()).setBorderPainted(false);
			}
		});
		aboutButton.setBorderPainted(false);
		aboutButton.setFocusable(false);
		aboutButton.setOpaque(false);
		aboutButton.setPreferredSize(new Dimension(32, 32));
		aboutButton.setToolTipText(StringConstants.getString(StringConstants.ABOUT_TOOLTIP));
		JPanel documentNorthPanel = guiFactory.createJPanel();
		documentNorthPanel.setLayout(new BoxLayout(documentNorthPanel, BoxLayout.X_AXIS));
		documentNorthPanel.add(guiFactory.createTitleLabel(StringConstants.getString(StringConstants.DOCUMENT_COLLECTION_LABEL)));
		documentNorthPanel.add(Box.createHorizontalGlue());
		documentNorthPanel.add(helpButton);
		documentNorthPanel.add(aboutButton);
		JButton documentButton = new JButton(StringConstants.getString(StringConstants.BROWSE_LABEL), Application.getIcon("browse.gif"));
		documentButton.setActionCommand("document");
		documentButton.addActionListener(control);
		subdirsCheckBox = new JCheckBox(StringConstants.getString(StringConstants.INCLUDE_SUBDIRECTORIES_LABEL), false);
		subdirsCheckBox.setOpaque(false);
		JPanel documentCenterPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.LEFT));
		documentCenterPanel.add(documentButton);
		documentCenterPanel.add(subdirsCheckBox);
		documentField = new JTextField(30);
		documentField.setEditable(false);
		documentField.setEnabled(false);
		documentField.setOpaque(false);
		JPanel documentSouthPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.LEFT));
		documentSouthPanel.add(documentField);
		documentPanel.add(documentNorthPanel, BorderLayout.NORTH);
		documentPanel.add(documentCenterPanel, BorderLayout.WEST);
		documentPanel.add(documentSouthPanel, BorderLayout.SOUTH);

		/*
		 * Index location panel
		 */
		JButton indexButton = new JButton(StringConstants.getString(StringConstants.BROWSE_LABEL), Application.getIcon("browse.gif"));
		indexButton.addActionListener(control);
		indexButton.setActionCommand("open");
		JPanel indexButtonPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.LEFT));
		indexButtonPanel.add(indexButton);
		indexField = new JTextField(30);
		indexField.setEditable(false);
		indexField.setEnabled(false);
		indexField.setOpaque(false);
		JPanel indexFieldPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.LEFT));
		indexFieldPanel.add(indexField);
		indexPanel.add(guiFactory.createTitleLabel(StringConstants.getString(StringConstants.INDEX_LOCATION_LABEL)), BorderLayout.NORTH);
		indexPanel.add(indexButtonPanel, BorderLayout.WEST);
		indexPanel.add(indexFieldPanel, BorderLayout.SOUTH);

		/*
		 * Parameter panel
		 */
		FontMetrics metrics = getFontMetrics(new Font("Dialog", Font.BOLD, 12));
		int labelWidth = 10 + Math.max(metrics.stringWidth(StringConstants.getString(StringConstants.LANGUAGE_LABEL)), Math.max(metrics.stringWidth(StringConstants.getString(StringConstants.STEMMING_LABEL)), metrics.stringWidth(StringConstants.getString(StringConstants.STOP_WORDS_LABEL))));
		JLabel languageLabel = new JLabel(StringConstants.getString(StringConstants.LANGUAGE_LABEL));
		languageLabel.setPreferredSize(new Dimension(labelWidth, 15));
		languageCombo = new JComboBox(new String[] { StringConstants.getString(StringConstants.LANGUAGE_ENGLISH), StringConstants.getString(StringConstants.LANGUAGE_GERMAN)});
		languageCombo.setActionCommand("language");
		languageCombo.addActionListener(control);
		JPanel parameterUpperPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.LEFT));
		parameterUpperPanel.setAlignmentX(LEFT_ALIGNMENT);
		parameterUpperPanel.add(languageLabel);
		parameterUpperPanel.add(languageCombo);
		JLabel stopWordLabel = new JLabel(StringConstants.getString(StringConstants.STOP_WORDS_LABEL));
		stopWordLabel.setPreferredSize(new Dimension(labelWidth, 15));
		stopWordCombo = new JComboBox(new String[] { StringConstants.getString(StringConstants.DEACTIVATED_LABEL), StringConstants.getString(StringConstants.PREDEFINED_LIST_LABEL), "benutzerdefiniert" });
		stopWordCombo.setActionCommand("stopWord");
		stopWordCombo.addActionListener(control);
		JPanel parameterMiddlePanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.LEFT));
		parameterMiddlePanel.setAlignmentX(LEFT_ALIGNMENT);
		parameterMiddlePanel.add(stopWordLabel);
		parameterMiddlePanel.add(stopWordCombo);
		JLabel stemmingLabel = new JLabel(StringConstants.getString(StringConstants.STEMMING_LABEL));
		stemmingLabel.setPreferredSize(new Dimension(labelWidth, 15));
		stemmingCombo = new JComboBox(new String[] { StringConstants.getString(StringConstants.DEACTIVATED_LABEL), StringConstants.getString(StringConstants.LINGUISTIC_LABEL)});
		stemmingCombo.setActionCommand("stemming");
		stemmingCombo.addActionListener(control);
		JPanel parameterLowerPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.LEFT));
		parameterLowerPanel.setAlignmentX(LEFT_ALIGNMENT);
		parameterLowerPanel.add(stemmingLabel);
		parameterLowerPanel.add(stemmingCombo);
		parameterPanel.add(guiFactory.createTitleLabel(StringConstants.getString(StringConstants.INDEX_PARAMETERS_LABEL)));
		parameterPanel.add(parameterUpperPanel);
		parameterPanel.add(parameterMiddlePanel);
		parameterPanel.add(parameterLowerPanel);

		/*
		 * Creation panel
		 */
		JButton createButton = new JButton(StringConstants.getString(StringConstants.CREATE_INDEX_LABEL), Application.getIcon("createIndex.gif"));
		createButton.setActionCommand("create");
		createButton.addActionListener(control);
		JButton showButton = new JButton(StringConstants.getString(StringConstants.SHOW_INDEX_LABEL), Application.getIcon("showIndex.gif"));
		showButton.setActionCommand("show");
		showButton.addActionListener(control);
		JPanel creationButtonPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.LEFT));
		creationButtonPanel.setAlignmentX(LEFT_ALIGNMENT);
		creationButtonPanel.add(createButton);
		creationButtonPanel.add(showButton);
		creationPanel.add(guiFactory.createTitleLabel(StringConstants.getString(StringConstants.INDEX_CREATION_LABEL)));
		creationPanel.add(creationButtonPanel);

		/*
		 * Query panel
		 */
		queryField = new JTextField(20);
		queryField.setBackground(UIManager.getColor("TextField.inactiveBackground"));
		queryField.setActionCommand("search");
		queryField.addActionListener(control);
		JButton searchButton = new JButton(StringConstants.getString(StringConstants.SEARCH_LABEL), Application.getIcon("search.gif"));
		searchButton.setActionCommand("search");
		searchButton.addActionListener(control);
		JPanel queryFieldPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.LEFT));
		queryFieldPanel.setAlignmentX(LEFT_ALIGNMENT);
		queryFieldPanel.add(queryField);
		queryFieldPanel.add(searchButton);
		JPanel rankingPanel = guiFactory.createJPanel();
		rankingPanel.setLayout(new BoxLayout(rankingPanel, BoxLayout.X_AXIS));
		rankingPanel.setAlignmentX(LEFT_ALIGNMENT);
		rankingPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 0, 5));
		JPanel column1Panel = guiFactory.createJPanel();
		column1Panel.setLayout(new BoxLayout(column1Panel, BoxLayout.Y_AXIS));
		JLabel label = new JLabel(StringConstants.getString(StringConstants.RANKING_PRINCIPLE_LABEL));
		label.setAlignmentX(CENTER_ALIGNMENT);
		column1Panel.add(label);
		column1Panel.add(Box.createRigidArea(new Dimension(0, 5)));
		JPanel tempPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.CENTER, 10, 0));
		tempPanel.add(new JLabel("1"));
		tempPanel.add(guiFactory.createLinkLabel("<html>[<font color=\"#0000ff\"><u>?</u></font>]</html>", StringConstants.getString(StringConstants.RANKING_PRINCIPLE_1_TOOLTIP), StringConstants.getString(StringConstants.RANKING_PRINCIPLE_LABEL)));
		column1Panel.add(tempPanel);
		column1Panel.add(Box.createRigidArea(new Dimension(0, 5)));
		tempPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.CENTER, 10, 0));
		tempPanel.add(new JLabel("2"));
		tempPanel.add(guiFactory.createLinkLabel("<html>[<font color=\"#0000ff\"><u>?</u></font>]</html>", StringConstants.getString(StringConstants.RANKING_PRINCIPLE_2_TOOLTIP), StringConstants.getString(StringConstants.RANKING_PRINCIPLE_LABEL)));
		column1Panel.add(tempPanel);
		column1Panel.add(Box.createRigidArea(new Dimension(0, 5)));
		tempPanel = guiFactory.createJPanel(new FlowLayout(FlowLayout.CENTER, 10, 0));
		tempPanel.add(new JLabel("3"));
		tempPanel.add(guiFactory.createLinkLabel("<html>[<font color=\"#0000ff\"><u>?</u></font>]</html>", StringConstants.getString(StringConstants.RANKING_PRINCIPLE_3_TOOLTIP), StringConstants.getString(StringConstants.RANKING_PRINCIPLE_LABEL)));
		column1Panel.add(tempPanel);
		ButtonGroup bg1 = new ButtonGroup();
		ButtonGroup bg2 = new ButtonGroup();
		ButtonGroup bg3 = new ButtonGroup();
		JPanel column2Panel = guiFactory.createJPanel();
		column2Panel.setLayout(new BoxLayout(column2Panel, BoxLayout.Y_AXIS));
		label = new JLabel(StringConstants.getString(StringConstants.DONT_CARE_LABEL));
		label.setAlignmentX(CENTER_ALIGNMENT);
		column2Panel.add(label);
		column2Panel.add(guiFactory.createJRadioButton(bg1, false, "radio1-0", control));
		column2Panel.add(guiFactory.createJRadioButton(bg2, false, "radio2-0", control));
		column2Panel.add(guiFactory.createJRadioButton(bg3, false, "radio3-0", control));
		JPanel column3Panel = guiFactory.createJPanel();
		column3Panel.setLayout(new BoxLayout(column3Panel, BoxLayout.Y_AXIS));
		label = new JLabel(StringConstants.getString(StringConstants.WEAK_LABEL));
		label.setAlignmentX(CENTER_ALIGNMENT);
		column3Panel.add(label);
		column3Panel.add(guiFactory.createJRadioButton(bg1, false, "radio1-1", control));
		column3Panel.add(guiFactory.createJRadioButton(bg2, false, "radio2-1", control));
		column3Panel.add(guiFactory.createJRadioButton(bg3, false, "radio3-1", control));
		JPanel column4Panel = guiFactory.createJPanel();
		column4Panel.setLayout(new BoxLayout(column4Panel, BoxLayout.Y_AXIS));
		label = new JLabel(StringConstants.getString(StringConstants.NORMAL_LABEL));
		label.setAlignmentX(CENTER_ALIGNMENT);
		column4Panel.add(label);
		column4Panel.add(guiFactory.createJRadioButton(bg1, true, "radio1-2", control));
		column4Panel.add(guiFactory.createJRadioButton(bg2, true, "radio2-2", control));
		column4Panel.add(guiFactory.createJRadioButton(bg3, true, "radio3-2", control));
		JPanel column5Panel = guiFactory.createJPanel();
		column5Panel.setLayout(new BoxLayout(column5Panel, BoxLayout.Y_AXIS));
		label = new JLabel(StringConstants.getString(StringConstants.STRONG_LABEL));
		label.setAlignmentX(CENTER_ALIGNMENT);
		column5Panel.add(label);
		column5Panel.add(guiFactory.createJRadioButton(bg1, false, "radio1-3", control));
		column5Panel.add(guiFactory.createJRadioButton(bg2, false, "radio2-3", control));
		column5Panel.add(guiFactory.createJRadioButton(bg3, false, "radio3-3", control));
		rankingPanel.add(column1Panel);
		rankingPanel.add(Box.createHorizontalGlue());
		rankingPanel.add(column2Panel);
		rankingPanel.add(Box.createHorizontalGlue());
		rankingPanel.add(column3Panel);
		rankingPanel.add(Box.createHorizontalGlue());
		rankingPanel.add(column4Panel);
		rankingPanel.add(Box.createHorizontalGlue());
		rankingPanel.add(column5Panel);
		queryPanel.add(guiFactory.createTitleLabel(StringConstants.getString(StringConstants.QUERY_LABEL)));
		queryPanel.add(queryFieldPanel);
		queryPanel.add(rankingPanel);

		/*
		 * Put it all together.
		 */
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(documentPanel);
		add(new JSeparator());
		add(indexPanel);
		add(new JSeparator());
		add(parameterPanel);
		add(new JSeparator());
		add(creationPanel);
		add(new JSeparator());
		add(queryPanel);
	}

	/**
	 * Creates a new index.
	 */
	public void createIndex() {
		if (documentDirectory != null && indexDirectory != null) {
			if (documentDirectory.exists() && indexDirectory.exists()) {
				try {
					//	Ask for deletion of existing files
					if (indexDirectory.listFiles().length > 0 && JOptionPane.showConfirmDialog(null, StringConstants.getString(StringConstants.OVERWRITE_INDEX_LABEL), StringConstants.getString(StringConstants.OVERWRITE_LABEL), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) != JOptionPane.OK_OPTION) {
						return;
					}

					//	Scan directory
					indexCanceled = false;
					indexWriter = new IndexWriter(indexDirectory, AnalyzerFactory.createAnalyzer(language, doStemming, removeStopWords, stopWords), true);
					fileVector = new Vector();
					File[] files = documentDirectory.listFiles();
					if (files != null) {
						for (int i = 0; i < files.length; i++) {
							scanDirectory(files[i]);
						}
					}
					if (fileVector.size() == 0) {
						JOptionPane.showMessageDialog(null, StringConstants.getString(StringConstants.NO_HTML_FILES_LABEL), StringConstants.getString(StringConstants.WARNING_LABEL), JOptionPane.WARNING_MESSAGE);
						return;
					}

					//	Create GUI for the progress bar
					final JProgressBar progressBar = new JProgressBar();
					progressBar.setStringPainted(true);
					final JLabel statusLabel = new JLabel("");
					JButton cancelButton = new JButton(StringConstants.getString(StringConstants.CANCEL_LABEL), Application.getIcon("cancel.gif"));
					cancelButton.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							Application.this.indexCanceled = true;
						}
					});
					JPanel buttonPanel = guiFactory.createJPanel();
					buttonPanel.add(cancelButton);
					JPanel panel = guiFactory.createJPanel(new BorderLayout(10, 10));
					panel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
					panel.add(statusLabel, BorderLayout.NORTH);
					panel.add(progressBar, BorderLayout.CENTER);
					panel.add(buttonPanel, BorderLayout.SOUTH);
					final JFrame frame = guiFactory.createJFrame(StringConstants.getString(StringConstants.CREATING_INDEX_LABEL), panel);
					frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
					frame.setSize(new Dimension(600, 180));
					Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
					frame.setLocation((screen.width - frame.getWidth()) / 2, (screen.height - frame.getHeight()) / 2);
					frame.show();

					//	Create the index task and start it
					final IndexTask task = new IndexTask(this);
					timer = new Timer(100, new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							progressBar.setValue(task.getCurrent());
							statusLabel.setText(task.getStatus());
							if (task.done()) {
								timer.stop();
								try {
									indexWriter.optimize();
									indexWriter.close();
								} catch (Exception ex) {
									ex.printStackTrace(System.err);
								}
								frame.dispose();
							}
						}
					});
					task.go();
					timer.start();

					return;
				} catch (Exception ex) {
					ex.printStackTrace(System.err);
				}
			}
		}
		JOptionPane.showMessageDialog(null, StringConstants.getString(StringConstants.SPECIFY_LABEL), StringConstants.getString(StringConstants.ERROR_LABEL), JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Adds the file to the file vector or scans it if it is a directory.
	 *
	 * @param	file	The file.
	 */
	protected void scanDirectory(File file) {
		if (file.isDirectory() && subdirsCheckBox.isSelected()) {
			File fileArray[] = file.listFiles();
			for (int i = 0; i < fileArray.length; i++) {
				scanDirectory(fileArray[i]);
			}
		} else if (file.isFile()) {
			String name = file.getName().toLowerCase();
			if (name.endsWith(".html") || name.endsWith(".htm")) {
				fileVector.add(file);
			}
		}
	}

	/**
	 * Starts a search.
	 */
	public void search() {
		try {
			//	Search the index for the given query
			String queryText = queryField.getText().replaceAll("\\+|\\-|\\&|\\||\\!|\\(|\\)|\\{|\\}|\\[|\\]|\\^|\"|~|\\*|\\?|\\:|\\\\", " ");
			Query query = QueryParser.parse(queryText, "contents", AnalyzerFactory.createAnalyzer(language, doStemming, removeStopWords, stopWords));
			IndexSearcher indexSearcher = new IndexSearcher(indexDirectory.getAbsolutePath());
			Hits hits = indexSearcher.search(query);

			if (hits.length() == 0) {
				JOptionPane.showMessageDialog(null, StringConstants.getString(StringConstants.NO_DOCUMENTS_LABEL), StringConstants.getString(StringConstants.RESULT_LABEL), JOptionPane.INFORMATION_MESSAGE);
			} else {
				//	Create a temporary HTML file for the search result
				String[] rankingWeights = { StringConstants.getString(StringConstants.DONT_CARE_LABEL), StringConstants.getString(StringConstants.WEAK_LABEL), StringConstants.getString(StringConstants.NORMAL_LABEL), StringConstants.getString(StringConstants.STRONG_LABEL)};
				String[] args = new String[2];
				args[0] = "" + hits.length();
				args[1] = (hits.length() == 1) ? "" : StringConstants.getString(StringConstants.DOCUMENTS_PLURAL_LABEL);
				File htmlFile = createTemporaryFile(StringConstants.RESULT_FILE_PREFIX, StringConstants.RESULT_FILE_SUFFIX);
				htmlFile.deleteOnExit();
				FileWriter writer = new FileWriter(htmlFile);
				writer.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n\n" + "<HTML>\n\n<HEAD>\n  <meta HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; CHARSET=ISO-8859-1\">\n" + "  <TITLE>" + StringConstants.getString(StringConstants.RESULT_LABEL) + ": " + queryText + "</TITLE>\n</HEAD>\n\n<BODY>\n\n");
				writer.write("  <P><FONT SIZE=5><B>" + StringConstants.getString(StringConstants.DOCUMENTS_FOUND_LABEL, args) + "</FONT><BR>\n" + "   " + StringConstants.getString(StringConstants.QUERY_LABEL) + ":&nbsp;&nbsp;&quot;" + queryText + "&quot;<BR>\n" + "   " + StringConstants.getString(StringConstants.RANKING_PRINCIPLE_LABEL) + ":&nbsp;&nbsp;1: " + rankingWeights[ranking1] + "&nbsp;&nbsp;2: " + rankingWeights[ranking2] + "&nbsp;&nbsp;3: " + rankingWeights[ranking3] + "</B></P>\n\n  <HR>\n");

				//	Write the result to the HTML file
				RankedDocumentSet docs = doRanking(query.toString("contents"), hits);
				for (int i = 0; i < docs.getSize(); i++) {
					Document doc = docs.getDocument(i);
					String url = (new File(doc.get("url"))).toURL().toString();
					writer.write("\n  <P><FONT SIZE=5><B>" + (i + 1) + ". " + doc.get("title") + "</B></FONT><BR>\n" + "   " + doc.get("summary") + "<BR>\n" + "   <A HREF=\"" + url + "\">" + doc.get("url") + "</A></P>\n");
				}
				writer.write("\n</BODY>\n\n</HTML>\n");
				writer.close();

				//	Display the HTML file containing the results
				Desktop.getDesktop().browse(htmlFile.toURI());
			}
			indexSearcher.close();
		} catch (OutOfMemoryError ex) {
			JOptionPane.showMessageDialog(null, StringConstants.getString(StringConstants.NOT_ENOUGH_MEMORY_LABEL), StringConstants.getString(StringConstants.ERROR_LABEL), JOptionPane.ERROR_MESSAGE);
		} catch (Exception ex) {
			if (indexDirectory != null && indexDirectory.exists() && (new File(indexDirectory.getAbsoluteFile() + File.separator + "soekia-index.xml")).exists()) {
				JOptionPane.showMessageDialog(null, StringConstants.getString(StringConstants.QUERY_ILLEGAL_LABEL), StringConstants.getString(StringConstants.ERROR_LABEL), JOptionPane.ERROR_MESSAGE);
			} else {
				JOptionPane.showMessageDialog(null, StringConstants.getString(StringConstants.NO_INDEX_LABEL), StringConstants.getString(StringConstants.ERROR_LABEL), JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	/**
	 * Calculates the ranking of the hits.
	 *
	 * @param	query	The query.
	 * @param	hits	The hits of the query.
	 *
	 * @return			The ranked document set.
	 */
	protected RankedDocumentSet doRanking(String query, Hits hits) throws Exception {
		RankedDocumentSet docs = new RankedDocumentSet(hits.length(), 3);

		for (int i = 0; i < hits.length(); i++) {
			docs.addDocument(hits.doc(i));
		}

		//	Retrieve term and document frequency
		ArrayList queryTerms = new ArrayList();
		IndexReader indexReader = IndexReader.open(indexDirectory);
		StringTokenizer tokenizer = new StringTokenizer(query);
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken();
			try {
				QueryTerm queryTerm = new QueryTerm(token);
				Term term = new Term("contents", token);
				TermDocs termDocs = indexReader.termDocs(term);
				int totalFrequency = 0;
				if (termDocs.next()) {
					int countDocuments = 1;
					queryTerm.docs.put(indexReader.document(termDocs.doc()).get(("url")), new Integer(termDocs.freq()));
					totalFrequency += termDocs.freq();
					while (termDocs.next()) {
						queryTerm.docs.put(indexReader.document(termDocs.doc()).get(("url")), new Integer(termDocs.freq()));
						totalFrequency += termDocs.freq();
						countDocuments++;
					}
					queryTerm.documentFrequency = countDocuments;
				}
				queryTerms.add(queryTerm);
			} catch (Exception ex) {
				ex.printStackTrace(System.err);
			}
		}
		indexReader.close();

		//	Calculate relevance
		double max1 = 0;
		double max2 = 0;
		double max3 = 0;
		for (int i = 0; i < queryTerms.size(); i++) {
			QueryTerm qt = (QueryTerm) queryTerms.get(i);
			Enumeration enumer = qt.docs.keys();
			while (enumer.hasMoreElements()) {
				String url = (String) enumer.nextElement();
				int pos = docs.getPosition(url);
				docs.setRelevance(pos, 0, 1 + docs.getRelevance(pos, 0));
				docs.setRelevance(pos, 1, ((Integer)qt.docs.get(url)).intValue() + docs.getRelevance(pos, 1));
				docs.setRelevance(pos, 2, 1.0 / qt.documentFrequency + docs.getRelevance(pos, 2));
				if (docs.getRelevance(pos, 0) > max1) {
					max1 = docs.getRelevance(pos, 0);
				}
				if (docs.getRelevance(pos, 1) > max2) {
					max2 = docs.getRelevance(pos, 1);
				}
				if (docs.getRelevance(pos, 2) > max3) {
					max3 = docs.getRelevance(pos, 2);
				}
			}
		}
		double factor = ranking1 / Math.max(ranking1 + ranking2 + ranking3, 1.0);
		if (max1 > 0) {
			for (int i = 0; i < docs.getSize(); i++) {
				docs.setRelevance(i, 0, factor * normalize(docs.getRelevance(i, 0) / max1));
			}
		}
		factor = ranking2 / Math.max(ranking1 + ranking2 + ranking3, 1.0);
		if (max2 > 0) {
			for (int i = 0; i < docs.getSize(); i++) {
				docs.setRelevance(i, 1, factor * normalize(docs.getRelevance(i, 1) / max2));
			}
		}
		factor = ranking3 / Math.max(ranking1 + ranking2 + ranking3, 1.0);
		if (max3 > 0) {
			for (int i = 0; i < docs.getSize(); i++) {
				docs.setRelevance(i, 2, factor * normalize(docs.getRelevance(i, 2) / max3));
			}
		}

		//	Sum up the relevances and sort the document set
		for (int i = 0; i < docs.getSize(); i++) {
			docs.setRelevance(i, 0, docs.getRelevance(i, 0) + docs.getRelevance(i, 1) + docs.getRelevance(i, 2));
		}
		docs.sort(0);

		return docs;
	}

	/**
	 * Normalizes the given value to the interval [0,1].
	 * 
	 * @param	value	The value
	 *
	 * @return			The value normalized to the interval [0,1].
	 */
	protected double normalize(double value) {
		return Math.sqrt(value * (2.0 - value));
	}

	/**
	 * Returns the current settings.
	 *
	 * @return		The current settings.
	 */
	public Settings getSettings() {
		Settings settings = new Settings("Soekia 1.0");
		settings.addSetting(new Setting("documentDirectory", documentDirectory.getAbsolutePath()));
		settings.addSetting(new Setting("language", language));
		settings.addSetting(new Setting("doStemming", doStemming ? "true" : "false"));
		settings.addSetting(new Setting("removeStopWords", removeStopWords ? "true" : "false"));

		return settings;
	}

	/**
	 * Saves the current settings.
	 */
	public void saveSettings() {
		XmlExporter exporter = new XmlExporter();
		exporter.exportSettings(getSettings(), new File(indexDirectory.getAbsolutePath() + File.separator + StringConstants.INDEX_FILE_NAME));
	}

	/**
	 * Sets the current settings.
	 *
	 * @param	settings	The new settings.
	 */
	public void setSettings(Settings settings) {
		if (settings == null) {
			return;
		}
		for (int i = 0; i < settings.getNumberSettings(); i++) {
			Setting setting = settings.getSetting(i);
			if (setting.name.equals("documentDirectory")) {
				setDocumentDirectory(new File((String) setting.values.get(0)));
			} else if (setting.name.equals("language")) {
				if (((String) setting.values.get(0)).equals(StringConstants.ENGLISH)) {
					languageCombo.setSelectedIndex(0);
				} else if (((String) setting.values.get(0)).equals(StringConstants.GERMAN)) {
					languageCombo.setSelectedIndex(1);
				} else {
					languageCombo.setSelectedIndex(1);
				}
			} else if (setting.name.equals("doStemming")) {
				String value = (String) setting.values.get(0);
				if (value.equals("false")) {
					stemmingCombo.setSelectedIndex(0);
				} else {
					stemmingCombo.setSelectedIndex(1);
				}
			} else if (setting.name.equals("removeStopWords")) {
				String value = (String) setting.values.get(0);
				if (value.equals("false")) {
					stopWordCombo.setSelectedIndex(0);
				} else {
					stopWordCombo.setSelectedIndex(1);
				}
			}
		}
	}

	/**
	 * Sets the document directory to the given directory.
	 *
	 * @param	file	The directory.
	 */
	public void setDocumentDirectory(File file) {
		documentDirectory = file;
		documentField.setEnabled(true);
		documentField.setText(file.getAbsolutePath());
	}

	/**
	 * Sets the index directory to the given directory.
	 *
	 * @param	file	The directory.
	 */
	public void setIndexDirectory(File file) {
		if (StringConstants.INDEX_DIRECTORY_NAME.equals(file.getName())) {
			indexDirectory = file;
		} else {
			indexDirectory = new File(file.getAbsoluteFile() + File.separator + StringConstants.INDEX_DIRECTORY_NAME);
		}
		if (!indexDirectory.exists()) {
			indexDirectory.mkdir();
		}
		indexField.setEnabled(true);
		indexField.setText(indexDirectory.getParent());

		File indexFile = new File(indexDirectory.getAbsolutePath() + File.separator + StringConstants.INDEX_FILE_NAME);
		if (file.exists()) {
			XmlImporter importer = new XmlImporter();
			setSettings(importer.importSettings(indexFile));
		}
	}

	/**
	 * Sets the language.
	 *
	 * @param	index	The index of the language.
	 */
	public void setLanguage(int index) {
		switch (index) {
			case 0 :
				language = StringConstants.ENGLISH;
				break;
			case 1 :
				language = StringConstants.GERMAN;
				break;
			default :
				break;
		}
	}

	/**
	 * Sets the stemming algorithm.
	 *
	 * @param	index	The index of the stemming algorithm.
	 */
	public void setStemming(int index) {
		switch (index) {
			case 0 :
				doStemming = false;
				break;
			case 1 :
				doStemming = true;
				if (stopWordCombo.getSelectedIndex() == 2) {
					stopWords = StringConstants.STOP_WORDS[languageCombo.getSelectedIndex()];
				}
				break;
			default :
				break;
		}
	}

	/**
	 * Sets the stop word algorithm.
	 *
	 * @param	index	The index of the stop word algorithm.
	 */
	public void setStopWord(int index) {
		switch (index) {
			case 0 :
				removeStopWords = false;
				break;
			case 1 :
				removeStopWords = true;
				stopWords = StringConstants.STOP_WORDS[languageCombo.getSelectedIndex()];
				break;
			case 2 :
				removeStopWords = true;
				showStopWordDialog();
				break;
			default :
				break;
		}
	}

	protected void showStopWordDialog() {
		stopWordDialog = new JDialog((Frame)null, StringConstants.getString(StringConstants.STOP_WORDS_TITLE), true);
		stopWordDialog.getContentPane().setLayout(new BorderLayout());
		JButton okButton = new JButton(StringConstants.getString(StringConstants.OK_LABEL));
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				stopWords = stopWordTextPane.getText();
				stopWordDialog.dispose();
			}
		});
		JButton cancelButton = new JButton(StringConstants.getString(StringConstants.CANCEL_LABEL));
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				stopWordDialog.dispose();
			}
		});
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);
		stopWordDialog.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		stopWordTextPane = new JEditorPane();
		stopWordTextPane.setFont(new Font("Dialog", Font.PLAIN, 14));
		stopWordTextPane.setText(stopWords);
		stopWordDialog.getContentPane().add(new JScrollPane(stopWordTextPane, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		stopWordDialog.setBounds((screen.width - 400)/2, (screen.height - 200)/2, 400, 200);
		stopWordDialog.show();
	}

	/**
	 * Sets the value of the ranking principle.
	 *
	 * @param	action	The action.
	 */
	public void setRanking(String action) {
		int value = Integer.parseInt(action.substring(action.indexOf('-') + 1));
		if (action.startsWith("radio1")) {
			ranking1 = value;
		} else if (action.startsWith("radio2")) {
			ranking2 = value;
		} else if (action.startsWith("radio3")) {
			ranking3 = value;
		}
	}

	/**
	 * Shows the 'About' dialog.
	 */
	public void showAboutDialog() {
		ImageIcon image = Application.getIcon(StringConstants.getString(StringConstants.SPLASH_FILE));
		JPanel aboutPanel = new JPanel(new BorderLayout());
		aboutPanel.add(new JLabel(image), BorderLayout.CENTER);
		JOptionPane.showMessageDialog(null, aboutPanel, StringConstants.getString(StringConstants.ABOUT_TITLE), JOptionPane.PLAIN_MESSAGE);
	}

	/**
	 * Shows the help window.
	 */
	public void showHelpWindow() {
		if (helpFrame == null) {
			helpFrame = new JFrame(StringConstants.getString(StringConstants.HELP_LABEL));
		}

		if (helpFrame.isShowing()) {
			helpFrame.toFront();
			return;
		}
		helpFrame.getContentPane().add(guiFactory.createHelpPanel());

		helpFrame.pack();
		helpFrame.setIconImage(Application.getIcon(StringConstants.getString(StringConstants.ICON_FILE)).getImage());
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		helpFrame.setLocation(screen.width / 2, (screen.height - helpFrame.getHeight()) / 2);
		helpFrame.show();
	}

	/**
	 * Displays the index in a new frame.
	 */
	public void showIndex() {
		try {
			//	Remove duplicate and bogus terms
			ArrayList termList = new ArrayList();
			Hashtable terms = new Hashtable();
			IndexReader indexReader = IndexReader.open(indexDirectory);
			TermEnum termEnum = indexReader.terms();
			String lastTerm = "";
			while (termEnum.next()) {
				Term term = termEnum.term();
				if (terms.containsKey(term.text()) || term.text().compareTo(lastTerm) < 0) {
					continue;
				}
				terms.put(term.text(), "");
				termList.add(term);
				lastTerm = term.text();
			}

			//	Create a temporary HTML file for the index
			File htmlFile = createTemporaryFile(StringConstants.INDEX_FILE_PREFIX, StringConstants.INDEX_FILE_SUFFIX);
			htmlFile.deleteOnExit();
			FileWriter writer = new FileWriter(htmlFile);
			writer.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n\n" + "<HTML>\n\n<HEAD>\n  <meta HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; CHARSET=ISO-8859-1\">\n" + "  <TITLE>" + StringConstants.getString(StringConstants.INDEX_LABEL) + " - " + indexDirectory.getParent() + "</TITLE>\n</HEAD>\n\n<BODY>\n\n");
			writer.write("  <TABLE>\n    <TR bgcolor=\"#BFBFBF\">\n" + "      <TD><B>" + StringConstants.getString(StringConstants.TERM_LABEL) + "</B></TD>\n" + "      <TD><B>" + StringConstants.getString(StringConstants.FREQUENCY_LABEL) + "</B></TD>\n" + "      <TD><B>" + StringConstants.getString(StringConstants.DOCUMENTS_LABEL) + "</B></TD>\n    </TR>\n");

			//	Iterate over the terms and gather information about the index
			Iterator iter = termList.iterator();
			int numberTerms = 0;
			while (iter.hasNext()) {
				String tableHeader = "    <TR bgcolor=\"#DFDFDF\">\n      <TD>{0}</TD>\n" + "      <TD>" + StringConstants.getString(StringConstants.TOTAL_LABEL) + ": {1}</TD>\n" + "      <TD>" + StringConstants.getString(StringConstants.DOCUMENTS_LABEL) + ": {2}</TD>\n    </TR>\n";
				String tableRow = "";
				Term term = (Term) iter.next();
				numberTerms++;
				try {
					TermDocs docs = indexReader.termDocs(term);
					int totalFrequency = 0;
					if (docs.next()) {
						int countDocuments = 1;
						tableRow = tableRow + "    <TR>\n      <TD>&nbsp;</TD>\n" + "      <TD ALIGN=\"RIGHT\">" + docs.freq() + "</TD>\n" + "      <TD><A HREF=\"" + (new File(indexReader.document(docs.doc()).get("url"))).toURL() + "\">" + indexReader.document(docs.doc()).get("url") + "</A></TD>\n    </TR>\n";
						totalFrequency += docs.freq();
						while (docs.next()) {
							tableRow = tableRow + "    <TR>\n      <TD>&nbsp;</TD>\n" + "      <TD ALIGN=\"RIGHT\">" + docs.freq() + "</TD>\n" + "      <TD><A HREF=\"" + (new File(indexReader.document(docs.doc()).get("url"))).toURL() + "\">" + indexReader.document(docs.doc()).get("url") + "</A></TD>\n    </TR>\n";
							totalFrequency += docs.freq();
							countDocuments++;
						}
						String[] args = new String[3];
						args[0] = term.text();
						args[1] = "" + totalFrequency;
						args[2] = "" + countDocuments;
						writer.write(MessageFormat.format(tableHeader, args));
						writer.write(tableRow);
					}
				} catch (Exception ex) {
					ex.printStackTrace(System.err);
				}
			}
			writer.write("  </TABLE>\n\n  <P>" + StringConstants.getString(StringConstants.TOTAL_TERMS_LABEL) + ": " + numberTerms + "</P>\n\n</BODY>\n\n</HTML>\n");
			writer.close();
			indexReader.close();

			//	Display the HTML file containing the index
			Desktop.getDesktop().browse(htmlFile.toURI());
		} catch (OutOfMemoryError ex) {
			JOptionPane.showMessageDialog(null, StringConstants.getString(StringConstants.NOT_ENOUGH_MEMORY_INDEX_LABEL), StringConstants.getString(StringConstants.ERROR_LABEL), JOptionPane.ERROR_MESSAGE);
		} catch (Exception ex) {
			JOptionPane.showMessageDialog(null, StringConstants.getString(StringConstants.NO_INDEX_DISPLAY_LABEL), StringConstants.getString(StringConstants.ERROR_LABEL), JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Creates a new temporary file for the given prefix and suffix.
	 *
	 * @param	prefix	The prefix of the file name.
	 * @param	suffix	The suffix of the file name.
	 *
	 * @return			A new temporary file for the given prefix and suffix or
	 * 					<code>null</code> if the file cannot be created.
	 */
	protected File createTemporaryFile(String prefix, String suffix) {
		try {
			File file = File.createTempFile(prefix, suffix);
			return file;
		} catch (Exception ex) {
			ex.printStackTrace(System.err);
			return null;
		}
	}

	/**
	 * Main method.
	 *
	 * @param	args	Select language: java -jar Soekia.jar [english|deutsch]
	 */
	public static void main(String[] args) {
		if (args.length > 0 && (args[0].equals("deutsch") || args[0].equals("english"))) {
			StringConstants.initialize(args[0]);
		} else {
			System.out.println("\nUSAGE: java -jar Soekia.jar [english|deutsch]\n");
			StringConstants.initialize("english");
		}

		//	Set look and feel
		try {
			JDialog.setDefaultLookAndFeelDecorated(true);
			JFrame.setDefaultLookAndFeelDecorated(true);
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
		} catch (Exception ex) {
			ex.printStackTrace(System.err);
		}

		//	Some Mac OS X properties to fix GUI problems
		System.setProperty("com.apple.mrj.application.growbox.intrudes", "false");
		System.setProperty("com.apple.mrj.application.live-resize", "true");

		JFrame frame = new JFrame(StringConstants.getString(StringConstants.APPLICATION_TITLE));
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		JPanel aboutPanel = new JPanel(new BorderLayout());
		ImageIcon image = Application.getIcon(StringConstants.getString(StringConstants.SPLASH_FILE));
		aboutPanel.add(new JLabel(image));
		aboutPanel.setBackground(Color.white);
		aboutPanel.setPreferredSize(new Dimension(image.getIconWidth(), image.getIconHeight()));

		JWindow splashWindow = new JWindow();
		splashWindow.getContentPane().add(aboutPanel);
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		splashWindow.pack();
		splashWindow.setLocation((screen.width - splashWindow.getWidth()) / 2, (screen.height - splashWindow.getHeight()) / 2);
		splashWindow.show();
		splashWindow.toFront();
		try {
			Thread.sleep(1500);
		} catch (Exception ex) {}

		Application app = new Application();
		frame.getContentPane().add(app);
		frame.pack();
		frame.setIconImage(Application.getIcon(StringConstants.getString(StringConstants.ICON_FILE)).getImage());
		frame.setLocation((screen.width - frame.getWidth()) / 2, (screen.height - frame.getHeight()) / 2);
		frame.setResizable(false);
		frame.show();
		splashWindow.dispose();
	}

	/**
	 * Returns the image icon with given name.
	 *
	 * @param	name	The name of the icon.
	 * @return			The image icon with given name or <code>null</code> if
	 * 					an error occured.
	 */
	protected static ImageIcon getIcon(String name) {
		try {
			ImageIcon icon = new ImageIcon(ClassLoader.getSystemResource("ch/educeth/soekia/resources/icons/" + name));
			return icon;
		} catch (Exception ex) {
			return null;
		}
	}

}
