/*
 *  ----------------------------------------------------------------------------
 *  Filename      : RankedDocumentSet.java
 *  ----------------------------------------------------------------------------
 *  Application   : Soekia
 *  Author        : Matthias Dreier
 *  File created  : 26.09.2003
 *  ----------------------------------------------------------------------------
 *  This file is part of Soekia.
 *
 *  Soekia is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Soekia is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Soekia; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  ----------------------------------------------------------------------------
 */
package ch.educeth.soekia.util;

import org.apache.lucene.document.Document;

/**
 * RankedDocumentSet.
 *
 * @author	Matthias Dreier
 */
public class RankedDocumentSet {

	/**
	 * The array of relevances.
	 */
	protected double relevance[][];

	/**
	 * The number of documents in the set.
	 */
	protected int count;

	/**
	 * The number of ranking principles.
	 */
	protected int numberRankingPrinciples;

	/**
	 * The array of document URLs.
	 */
	protected String[] documentUrl;

	/**
	 * The array of documents.
	 */
	protected Document[] docs;

	/**
	 * Creates a new ranked document set for the given number of documents and
	 * ranking principles.
	 *
	 * @param	numberDocuments				The number of documents.
	 * @param	numberRankingPrinciples		The number of ranking principles.
	 */
	public RankedDocumentSet(int numberDocuments, int numberRankingPrinciples) {
		count = 0;
		this.numberRankingPrinciples = numberRankingPrinciples;
		docs = new Document[numberDocuments];
		documentUrl = new String[numberDocuments];
		relevance = new double[numberDocuments][numberRankingPrinciples];
	}

	/**
	 * Inserts the given document.
	 *
	 * @param	url		The document.
	 */
	public void addDocument(Document document) {
		if (count < documentUrl.length) {
			docs[count] = document;
			documentUrl[count] = document.get("url");
			count++;
		}
	}

	/**
	 * Returns the capacity of this set.
	 *
	 * @return		The capacity of this set.
	 */
	public int getCapacity() {
		return documentUrl.length;
	}

	/**
	 * Returns the document at the given position.
	 *
	 * @param	position	The position of the document.
	 *
	 * @return				The document at the given position or
	 * 						<code>null</code> if the document does not exist.
	 */
	public Document getDocument(int position) {
		if (position >= 0 && position < count) {
			return docs[position];
		}

		return null;
	}

	/**
	 * Returns the position of the document with given URL.
	 *
	 * @param	url		The URL of the document.
	 *
	 * @return			The position of the document with given URL or
	 * 					<code>-1</code> if the document does not exist.
	 */
	public int getPosition(String url) {
		for (int i = 0; i < count; i++) {
			if (documentUrl[i].equals(url)) {
				return i;
			}
		}

		return -1;
	}

	/**
	 * Returns the relevance of the document at the given position according to
	 * the given ranking principle.
	 *
	 * @param	position			The position of the document
	 * @param	rankingPrinciple	The ranking principle.
	 *
	 * @return						The relevance of the document at the given
	 * 								position according to the given ranking
	 * 								principle or <code>0.0</code> if the
	 * 								document does not exist.
	 */
	public double getRelevance(int position, int rankingPrinciple) {
		if (position >= 0 && position < count) {
			return relevance[position][rankingPrinciple];
		}

		return 0.0;
	}

	/**
	 * Returns the number of documents in this set.
	 *
	 * @return		The number of documents in this set.
	 */
	public int getSize() {
		return count;
	}

	/**
	 * Return the URL of the document at the given position.
	 *
	 * @param	position	The position of the document.
	 *
	 * @return				The URL of the document at the given position or the
	 * 						empty string if the document does not exist.
	 */
	public String getURL(int position) {
		if (position >= 0 && position < count) {
			return documentUrl[position];
		}

		return "";
	}

	/**
	 * Sets the relevance of the document at the given position according to the
	 * given ranking principle to the given value.
	 *
	 * @param	position			The position of the document.
	 * @param	rankingPrinciple	The ranking principle.
	 * @param	value				The relevance value.
	 */
	public void setRelevance(int position, int rankingPrinciple, double value) {
		if (position >= 0 && position < count) {
			relevance[position][rankingPrinciple] = value;
		}
	}

	/**
	 * Sorts the documents according to the given ranking principle.
	 *
	 * @param	rankingPrinciple	The ranking principle.
	 */
	public void sort(int rankingPrinciple) {
		for (int i = 1; i < count; i++) {
			if (relevance[i - 1][rankingPrinciple] < relevance[i][rankingPrinciple]) {
				Document doc = docs[i];
				docs[i] = docs[i - 1];
				docs[i - 1] = doc;
				String url = documentUrl[i];
				documentUrl[i] = documentUrl[i - 1];
				documentUrl[i - 1] = url;
				for (int j = 0; j < numberRankingPrinciples; j++) {
					double value = relevance[i][j];
					relevance[i][j] = relevance[i - 1][j];
					relevance[i - 1][j] = value;
				}
				i = 0;
			}
		}
	}

}
